<?php

/**
 * Retrieve a sign-in token.
 */
function pm_get_token($username, $is_reseller)
{
    $api_root = "https://client-portal.patchman.co/api/v1";

    $fields = array('username' => $username);

    if ($is_reseller) {
        $fields['next'] = "/detections/reseller/";
    }

    try {
        // First, try contacting the SSO socket on a locally running patchmand
        $request_json = json_encode($fields);
        $response_json = "";

        $their_path = "/var/run/patchman-sso.sock";
        if (!file_exists($their_path)) {
            throw new Exception("sso socket does not exist");
        }

        $our_path = tempnam("/tmp", "patchman");
        if ($our_path === false) {
            throw new Exception("could not create temporary file");
        }

        unlink($our_path);

        $socket = socket_create(AF_UNIX, SOCK_DGRAM, 0);

        if ($socket !== false &&
            file_exists($their_path) &&
            socket_bind($socket, $our_path) &&
            socket_sendto($socket, $request_json, strlen($request_json), 0, $their_path) !== false &&
            socket_recvfrom($socket, $response_json, 8096, 0, $their_path) !== false &&
            unlink($our_path)
        ) {
            $data = json_decode($response_json);
        } else {
            throw new Exception(socket_strerror(socket_last_error()));
        }
    } catch (Exception $e) {
        // If something went wrong, attempt contacting the portal directly
        $c = curl_init();

        curl_setopt($c, CURLOPT_URL, "$api_root/token/");
        curl_setopt($c, CURLOPT_SSLCERT, "/etc/patchman/license/patchman.crt");
        curl_setopt($c, CURLOPT_SSLKEY, "/etc/patchman/license/patchman.key");
        curl_setopt($c, CURLOPT_POSTFIELDS, $fields);
        curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);

        if (function_exists('openssl_x509_parse')) {
            // False warnings from https://bugs.php.net/bug.php?id=66636 will interfere with output HTML, so silence the output
            $certificate_info = @openssl_x509_parse(file_get_contents('/etc/patchman/license/patchman.crt'));

            if ($certificate_info) {
                curl_setopt($c, CURLOPT_INTERFACE, $certificate_info['subject']['CN']);
            }
        }

        $version = curl_version();

        if (strpos($version['ssl_version'], "NSS/") === 0) {
            curl_setopt($c, CURLOPT_CAINFO, "/etc/patchman/license/patchman.crt");

            if (file_exists("/etc/pki/tls/")) {
                curl_setopt($c, CURLOPT_CAPATH, "/etc/pki/tls/");
            } else {
                curl_setopt($c, CURLOPT_CAPATH, "/etc/ssl/certs/");
            }
        }

        $data = json_decode(curl_exec($c));

        curl_close($c);
    }

    return $data;
}
